@@ -125,6 +125,7 @@ class Workflow(Base):
         if loop is None:
             loop = {
                 'max_iterations': 1,
+                'groups': [],
             }
 
         # Command group directories must be stored internally as variables:
@@ -264,7 +265,9 @@ class Workflow(Base):
 
     @property
     def all_alternate_scratch(self):
-        return [i.alternate_scratch for i in self.command_groups if i.alternate_scratch]
+        out = list(set([i.alternate_scratch for i in self.command_groups
+                        if i.alternate_scratch]))
+        return out
 
     @property
     def directory(self):
@@ -473,7 +476,7 @@ class Workflow(Base):
                                       self.root_archive_directory)
 
     def get_stats(self, jsonable=True, datetime_dicts=False):
-        'Get task statistics for this workflow.'
+        """Get task statistics for this workflow."""
         out = {
             'workflow_id': self.id_,
             'submissions': [i.get_stats(jsonable=jsonable, datetime_dicts=datetime_dicts)
@@ -482,15 +485,18 @@ class Workflow(Base):
         return out
 
     def kill_active(self):
-        'Kill any active scheduled jobs associated with the workflow.'
+        """Kill any active scheduled jobs associated with the workflow."""
 
         kill_scheduler_ids = []
         for sub in self.submissions:
             for cg_sub in sub.command_group_submissions:
                 for iteration in self.iterations:
                     cg_sub_iter = cg_sub.get_command_group_submission_iteration(iteration)
-                    if cg_sub_iter.scheduler_job_id is not None:
-                        kill_scheduler_ids.append(cg_sub_iter.scheduler_job_id)
+                    if cg_sub_iter:
+                        if cg_sub_iter.scheduler_job_id is not None:
+                            kill_scheduler_ids.append(cg_sub_iter.scheduler_job_id)
+                        if cg_sub_iter.scheduler_stats_job_id is not None:
+                            kill_scheduler_ids.append(cg_sub_iter.scheduler_stats_job_id)
 
         print('Need to kill: {}'.format(kill_scheduler_ids))
         del_cmd = ['qdel'] + [str(i) for i in kill_scheduler_ids]
@@ -651,7 +657,7 @@ class CommandGroup(Base):
 
     @staticmethod
     def get_command_lines(commands):
-        'Get all lines in the commands list.'
+        """Get all lines in the commands list."""
         out = []
         for i in commands:
             if 'line' in i:
@@ -982,6 +988,7 @@ class Submission(Base):
                 cg_sub_iter = CommandGroupSubmissionIteration(iteration, cg_sub)
                 cg_sub_iters.append(cg_sub_iter)
 
+        session.commit()
         # `cg_sub_iter.num_outputs` requires all cg_sub_iters to be generated:
         for cg_sub_iter in cg_sub_iters:
             for task_num in range(cg_sub_iter.num_outputs):
@@ -994,7 +1001,7 @@ class Submission(Base):
         self._scheduler_groups = self.get_scheduler_groups()
 
     def _make_alternate_scratch_dirs(self):
-        'Create a new directory on each alternate scratch for this submission.'
+        """Create a new directory on each alternate scratch for this submission."""
 
         alt_scratches = self.workflow.all_alternate_scratch
 
@@ -1038,7 +1045,7 @@ class Submission(Base):
         return self._scheduler_groups
 
     def get_scheduler_groups(self):
-        'Get scheduler groups for this workflow submission.'
+        """Get scheduler groups for this workflow submission."""
         return SchedulerGroup.get_scheduler_groups(self)
 
     def get_scheduler_group_index(self, command_group_submission):
@@ -1226,6 +1233,7 @@ class Submission(Base):
         submit_path = wf_path.joinpath('submit_{}'.format(self.order_id))
         submit_path.mkdir()
 
+        num_dir_vals_first_iter = {}  # keys are cg_sub_idx
         for iteration in self.workflow.iterations:
 
             # Make the iteration directory for each iteration:
@@ -1245,8 +1253,13 @@ class Submission(Base):
                 for cg_sub_idx, cg_sub in enumerate(i.command_group_submissions):
 
                     cg_sub_iter = cg_sub.get_command_group_submission_iteration(iteration)
+                    if not cg_sub_iter:
+                        continue
 
-                    num_dir_vals = cg_sub_iter.num_directories
+                    cg_sub_first_iter = cg_sub.get_command_group_submission_iteration(
+                        self.first_iteration
+                    )
+                    num_dir_vals = cg_sub_first_iter.num_directories
                     all_dir_slots = [''] * max_num_tasks
 
                     # Distribute dirs over num_dir_slots:
@@ -1285,52 +1298,71 @@ class Submission(Base):
 
     def submit_jobscripts(self, jobscript_paths):
 
-        js_paths, js_stat_paths = jobscript_paths
+        loop_groups = self.workflow.loop['groups']
+        cmd_group_idx = range(len(self.workflow.command_groups))
+
+        if loop_groups:
+
+            pre_loop_idx = [i for i in cmd_group_idx if i < min(loop_groups)]
+            post_loop_idx = [i for i in cmd_group_idx if i > max(loop_groups)]
+
+            # List of tuples mapping jobscript path index (i.e. command group order id) to
+            # iteration index:
+            js_submissions = [(i, 0) for i in pre_loop_idx]
+
+            for iteration in self.workflow.iterations:
+                for i in loop_groups:
+                    js_submissions.append((i, iteration.order_id))
+
+            for i in post_loop_idx:
+                js_submissions.append((i, 0))
+
+        else:
+            js_submissions = [(i, 0) for i in cmd_group_idx]
 
         sumbit_cmd = os.getenv('HPCFLOW_QSUB_CMD', 'qsub')
         last_submit_id = None
-        for iteration in self.workflow.iterations:
+        js_paths, js_stat_paths = jobscript_paths
 
-            iter_idx_var = 'ITER_IDX={}'.format(iteration.order_id)
+        for cg_sub_idx, iter_idx in js_submissions:
 
-            for js_path_i, js_stat_path_i, cg_sub in zip(
-                    js_paths, js_stat_paths, self.command_group_submissions):
+            iter_idx_var = 'ITER_IDX={}'.format(iter_idx)
+            cg_sub = self.command_group_submissions[cg_sub_idx]
+            iteration = self.workflow.iterations[iter_idx]
+            cg_sub_iter = cg_sub.get_command_group_submission_iteration(iteration)
+            js_path_i, js_stat_path_i = js_paths[cg_sub_idx], js_stat_paths[cg_sub_idx]
 
-                cg_sub_iter = cg_sub.get_command_group_submission_iteration(iteration)
-                if cg_sub_iter is None:
-                    continue
+            qsub_cmd = [sumbit_cmd]
 
-                qsub_cmd = [sumbit_cmd]
+            if last_submit_id:
 
-                if last_submit_id:
+                # Add conditional submission:
+                if iteration.order_id > 0:
+                    hold_arg = '-hold_jid'
+                elif cg_sub.command_group.nesting == NestingType('hold'):
+                    hold_arg = '-hold_jid'
+                else:
+                    hold_arg = '-hold_jid_ad'
 
-                    # Add conditional submission:
-                    if iteration.order_id > 0:
-                        hold_arg = '-hold_jid'
-                    elif cg_sub.command_group.nesting == NestingType('hold'):
-                        hold_arg = '-hold_jid'
-                    else:
-                        hold_arg = '-hold_jid_ad'
+                qsub_cmd += [hold_arg, last_submit_id]
 
-                    qsub_cmd += [hold_arg, last_submit_id]
+            qsub_cmd += ['-v', iter_idx_var]
+            qsub_cmd.append(str(js_path_i))
 
-                qsub_cmd += ['-v', iter_idx_var]
-                qsub_cmd.append(str(js_path_i))
+            # Submit the jobscript:
+            job_id_str = self.submit_jobscript(qsub_cmd, js_path_i, iteration)
+            cg_sub_iter.scheduler_job_id = int(job_id_str)
+            last_submit_id = job_id_str
 
-                # Submit the jobscript:
-                job_id_str = self.submit_jobscript(qsub_cmd, js_path_i, iteration)
-                cg_sub_iter.scheduler_job_id = int(job_id_str)
+            # Submit the stats jobscript:
+            if js_stat_path_i:
+                st_cmd = [sumbit_cmd, '-hold_jid_ad', last_submit_id, '-v', iter_idx_var]
+                st_cmd.append(str(js_stat_path_i))
+
+                job_id_str = self.submit_jobscript(st_cmd, js_stat_path_i, iteration)
+                cg_sub_iter.scheduler_stats_job_id = int(job_id_str)
                 last_submit_id = job_id_str
 
-                # Submit the stats jobscript:
-                if js_stat_path_i:
-                    st_cmd = [sumbit_cmd, '-hold_jid_ad',
-                              last_submit_id, '-v', iter_idx_var]
-                    st_cmd.append(str(js_stat_path_i))
-
-                    job_id_str = self.submit_jobscript(st_cmd, js_stat_path_i, iteration)
-                    last_submit_id = job_id_str
-
     def submit_jobscript(self, cmd, js_path, iteration):
 
         cwd = str(self.workflow.directory)
@@ -1355,7 +1387,7 @@ class Submission(Base):
         return job_id_str
 
     def get_stats(self, jsonable=True, datetime_dicts=False):
-        'Get task statistics for this submission.'
+        """Get task statistics for this submission."""
         out = {
             'submission_id': self.id_,
             'command_group_submissions': [
@@ -1479,12 +1511,13 @@ class CommandGroupSubmission(Base):
 
     @property
     def scheduler_group(self):
-        'Get the scheduler group to which this command group belongs.'
+        """Get the scheduler group to which this command group belongs."""
         return self.submission.get_scheduler_group(self)
 
     def get_command_group_submission_iteration(self, iteration):
-        for i in iteration.command_group_submission_iterations:
-            if i.command_group_submission == self:
+
+        for i in self.command_group_submission_iterations:
+            if i.iteration == iteration:
                 return i
 
     @property
@@ -1769,7 +1802,7 @@ class CommandGroupSubmission(Base):
             handle.write(cmd_lns)
 
     def write_alt_scratch_exclusion_list(self, project, task, iteration):
-        'Write alternate scratch exclusion files (for e.g. rsync)'
+        """Write alternate scratch exclusion files (for e.g. rsync)"""
 
         # List of Paths to exclude, relative to `self.submission.workflow.directory`:
         excluded_paths = [
@@ -1815,11 +1848,11 @@ class CommandGroupSubmission(Base):
                 handle.write(str(exc_path) + '\n')
 
     def make_alternate_scratch_dirs(self, project, iteration):
-        'Generate task working directories on the alternate scratch.'
+        """Generate task working directories on the alternate scratch."""
 
         # Get task working directories:
-        working_dirs = [task.get_working_directory()
-                        for task in self.tasks if task.iteration == iteration]
+        cg_sub_iter = self.get_command_group_submission_iteration(iteration)
+        working_dirs = [task.get_working_directory() for task in cg_sub_iter.tasks]
 
         alt_scratch_root = self.command_group.alternate_scratch.joinpath(
             self.submission.alt_scratch_dir_name)
@@ -1829,7 +1862,7 @@ class CommandGroupSubmission(Base):
                 # Already made "root" dir.
                 continue
             alt_scratch_w_dir = alt_scratch_root.joinpath(working_dir.value)
-            alt_scratch_w_dir.mkdir(parents=True, exist_ok=False)
+            alt_scratch_w_dir.mkdir(parents=True, exist_ok=True)
 
     def get_iteration(self, iter_idx):
         for i in self.submission.workflow.iterations:
@@ -1876,7 +1909,7 @@ class CommandGroupSubmission(Base):
         self.command_group.archive.execute_with_lock(task)
 
     def get_stats(self, jsonable=True, datetime_dicts=False):
-        'Get task statistics for this command group submission.'
+        """Get task statistics for this command group submission."""
         out = {
             'command_group_submission_id': self.id_,
             'command_group_id': self.command_group.id_,
@@ -1976,7 +2009,7 @@ class IsCommandWriting(Base):
 
 
 class Task(Base):
-    'Class to represent a single task.'
+    """Class to represent a single task."""
 
     __tablename__ = 'task'
 
@@ -2038,7 +2071,7 @@ class Task(Base):
 
     @property
     def scheduler_id(self):
-        'Get the task ID, as understood by the scheduler.'
+        """Get the task ID, as understood by the scheduler."""
         num_tasks = self.command_group_submission_iteration.num_outputs
         step_size = self.command_group_submission_iteration.step_size
         scheduler_range = range(1, 1 + (num_tasks * step_size), step_size)
@@ -2078,7 +2111,7 @@ class Task(Base):
             return None
 
     def get_working_directory(self):
-        'Get the "working directory" of this task.'
+        """Get the "working directory" of this task."""
         dir_vals = self.command_group_submission_iteration.get_directories()
         dirs_per_task = len(dir_vals) / \
             self.command_group_submission_iteration.num_outputs
@@ -2091,7 +2124,7 @@ class Task(Base):
         return self.get_working_directory().value
 
     def get_stats(self, jsonable=True, datetime_dicts=False):
-        'Get statistics for this task.'
+        """Get statistics for this task."""
         out = {
             'task_id': self.id_,
             'order_id': self.order_id,
@@ -2252,7 +2285,7 @@ class Task(Base):
 
 
 class Iteration(Base):
-    'Class to represent a workflow iteration.'
+    """Class to represent a workflow iteration."""
 
     __tablename__ = 'iteration'
 
@@ -2273,11 +2306,13 @@ class Iteration(Base):
     def __repr__(self):
         out = (
             '{}('
+            'id={}, '
             'workflow_id={}, '
             'order_id={}'
             ')'
         ).format(
             self.__class__.__name__,
+            self.id_,
             self.workflow_id,
             self.order_id,
         )
@@ -2292,6 +2327,7 @@ class CommandGroupSubmissionIteration(Base):
     working_dirs_written = Column(Boolean, default=False)
     iteration_id = Column(Integer, ForeignKey('iteration.id'))
     scheduler_job_id = Column(Integer, nullable=True)
+    scheduler_stats_job_id = Column(Integer, nullable=True)
     command_group_submission_id = Column(
         Integer, ForeignKey('command_group_submission.id'))
 
@@ -2315,13 +2351,15 @@ class CommandGroupSubmissionIteration(Base):
             '{}('
             'iteration_id={}, '
             'command_group_submission_id={}, '
-            'scheduler_job_id={}'
+            'scheduler_job_id={}, '
+            'scheduler_stats_job_id={}'
             ')'
         ).format(
             self.__class__.__name__,
             self.iteration_id,
             self.command_group_submission_id,
             self.scheduler_job_id,
+            self.scheduler_stats_job_id,
         )
         return out
 
@@ -2391,14 +2429,16 @@ class CommandGroupSubmissionIteration(Base):
 
     @property
     def num_outputs(self):
-        'Get the number of outputs for this command group submission.'
-        return self.command_group_submission.scheduler_group.get_num_outputs(self.iteration)[
+        """Get the number of outputs for this command group submission."""
+        iteration = self.command_group_submission.submission.workflow.first_iteration
+        return self.command_group_submission.scheduler_group.get_num_outputs(iteration)[
             self.command_group_submission.scheduler_group_index[1]]
 
     @property
     def step_size(self):
-        'Get the scheduler step size for this command group submission.'
-        return self.command_group_submission.scheduler_group.get_step_size(self.iteration)[
+        """Get the scheduler step size for this command group submission."""
+        iteration = self.command_group_submission.submission.workflow.first_iteration
+        return self.command_group_submission.scheduler_group.get_step_size(iteration)[
             self.command_group_submission.scheduler_group_index[1]]
 
     @property
@@ -2406,7 +2446,7 @@ class CommandGroupSubmissionIteration(Base):
         return len(self.tasks)
 
     def write_working_directories(self, project):
-        'Replace lines in the working_dirs files with actual directory paths.'
+        """Replace lines in the working_dirs files with actual directory paths."""
 
         dir_vals = self.get_directories()
 
@@ -2532,7 +2572,7 @@ class SchedulerGroup(object):
 
     @classmethod
     def get_scheduler_groups(cls, submission):
-        'Split the command group submissions up into scheduler groups.'
+        """Split the command group submissions up into scheduler groups."""
 
         cmd_groups_split = []
         sch_group_idx = 0
